/**
 * 
 */
package com.rjconsultores.ventaboletos.relatorios.impl;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import com.rjconsultores.ventaboletos.relatorios.utilitarios.ArrayDataSource;
import com.rjconsultores.ventaboletos.relatorios.utilitarios.Relatorio;
import com.rjconsultores.ventaboletos.web.utilerias.NamedParameterStatement;

/**
 * @author Bruno H. G. Gouva <bruno@rjconsultores.com.br>
 * 
 */
public class RelatorioAcompanhamentoEquivalentes extends Relatorio {

	private static final Integer CEM = Integer.valueOf(100);
	private static final Integer MAX_DECIMAL = Integer.valueOf(2);

	enum IndicadorRelatorio {

		MPE(1), RECEITA_KM(2), RECEITA_VIAGEM(3), IAP(4), PAXKM(5), ABSOLUTO(6), EQ(7);

		public final int valor;

		IndicadorRelatorio(int valorOpcao) {
			valor = valorOpcao;
		}

		public static IndicadorRelatorio fromInt(Integer value) {
			if (value != null) {
				for (IndicadorRelatorio b : IndicadorRelatorio.values()) {
					if (value.equals(b.valor)) {
						return b;
					}
				}
			}
			return null;
		}
	}

	/**
	 * @param parametros
	 * @param conexao
	 * @throws Exception
	 */
	public RelatorioAcompanhamentoEquivalentes(Map<String, Object> parametros, Connection conexao) throws Exception {
		super(parametros, conexao);

		this.setCustomDataSource(new ArrayDataSource(this) {

			protected void prepareQuery() throws SQLException {

				Connection conexao = this.relatorio.getConexao();
				Map<String, Object> parametros = this.relatorio.getParametros();
				String sql = getSql();

				NamedParameterStatement stmt = new NamedParameterStatement(conexao, sql);

				if (parametros.get("EMPRESA_ID") != null)
					stmt.setInt("EMPRESA_ID", (Integer) parametros.get("EMPRESA_ID"));
				else
					stmt.setNull("EMPRESA_ID", java.sql.Types.INTEGER);

				if (parametros.get("TIPOSERVICIO_ID") != null)
					stmt.setInt("TIPOSERVICIO_ID", (Integer) parametros.get("TIPOSERVICIO_ID"));
				else
					stmt.setNull("TIPOSERVICIO_ID", java.sql.Types.INTEGER);

				stmt.setDate("DATA_MES", new java.sql.Date(((Date) parametros.get("DATA_MES")).getTime()));

				this.resultSet = stmt.executeQuery();
			}

			@Override
			public void initDados() throws Exception {

				this.prepareQuery();

				Date dataInicial = (Date) this.relatorio.getParametros().get("DATA_MES");

				while (this.resultSet.next()) {

					Integer rolOperativoId = this.resultSet.getInt("ROLOPERATIVO_ID");
					Integer corridaId = this.resultSet.getInt("CORRIDA_ID");
					Integer assentos = this.resultSet.getInt("ASSENTOS");

					Map<String, Object> row = new HashMap<String, Object>();
					BigDecimal totalMes = BigDecimal.ZERO;
					Integer totalDias = 0;

					Calendar cal = Calendar.getInstance();
					cal.setTime(dataInicial);
					cal.set(Calendar.DATE, 1);

					// Roda todos os dias do mes
					for (int dia = 1; dia <= cal.getActualMaximum(Calendar.DATE); dia++) {

						BigDecimal valor = getValorByIndicador(cal.getTime(), corridaId, rolOperativoId, (Integer) this.relatorio.getParametros().get("INDICADOR"), assentos);

						if (valor != null) {
							totalMes = totalMes.add(valor);
							totalDias++;
						}

						BigDecimal ZERO = new BigDecimal(0);
						if (valor != null && !valor.equals(ZERO)) {
							row.put("LINHA", this.resultSet.getString("SIGLA"));
							row.put("LOTACAO", this.resultSet.getInt("ASSENTOS"));
							row.put("SERVICO", this.resultSet.getString("TIPO_SERVICO"));
							row.put("CODIGO", this.resultSet.getInt("CORRIDA_ID"));
							row.put("HORARIO", this.resultSet.getString("HORARIO"));
							row.put("INTERESTADUAL", this.resultSet.getString("INTERESTADUAL"));
							row.put("GRUPO_LINHA", this.resultSet.getString("GRUPO_LINHA"));

							row.put(String.valueOf(cal.get(Calendar.DATE)), valor);
						}

						if (cal.get(Calendar.DATE) < cal.getActualMaximum(Calendar.DATE))
							cal.add(Calendar.DATE, 1);
					}

					if (totalMes != null && !totalMes.equals(BigDecimal.ZERO) && totalDias > 0) {
						row.put("MEDIA", totalMes.divide(BigDecimal.valueOf(totalDias), 2, RoundingMode.HALF_UP));

						this.dados.add(row);
					}
				}

			}

			protected BigDecimal getValorByIndicador(Date data, Integer corridaId, Integer rolOperativoId, Integer indicador, Integer assentos) throws SQLException {

				String sql = null;

				switch (IndicadorRelatorio.fromInt(indicador)) {
				case MPE:
					sql = getSqlIndicadorEquivalentes();
					break;
				case ABSOLUTO:
					sql = getSqlIndicadorAbsoluto();
					break;
				case EQ:
					sql = getSqlIndicadorEq();
					break;
				case IAP:
					sql = getSqlIndicadorIap();
					break;
				case PAXKM:
					sql = getSqlIndicadorPax();
					break;
				case RECEITA_KM:
					sql = getSqlIndicadorReceitaKm();
					break;
				case RECEITA_VIAGEM:
					sql = getSqlIndicadorReceitaViagem();
					break;
				}

				return getValorIndicador(data, corridaId, rolOperativoId, indicador, assentos, sql);
			}

		});
	}

	protected String getSqlIndicadorEq() {
		StringBuilder sql = new StringBuilder();
		sql.append(" SELECT ROUND((SELECT TF.PRECIO");
		sql.append("                 FROM CORRIDA CR, TRAMO TR, TARIFA TF, VIGENCIA_TARIFA VT");
		sql.append("                WHERE CR.CORRIDA_ID = :CORRIDA_ID");
		sql.append("                  AND CR.FECCORRIDA = :FECCORRIDA");
		sql.append("                  AND CR.ROLOPERATIVO_ID = :ROLOPERATIVO_ID");
		sql.append("                  AND TF.CLASESERVICIO_ID = CR.CLASESERVICIO_ID");
		sql.append("                  AND TF.MARCA_ID = CR.MARCA_ID");
		sql.append("                  AND TF.RUTA_ID = CR.RUTA_ID");
		sql.append("                  AND TF.TRAMO_ID = TR.TRAMO_ID");
		sql.append("                  AND TF.STATUSTARIFA = 'A'");
		sql.append("                  AND TF.ACTIVO = 1");
		sql.append("                  AND TR.ORIGEN_ID = CR.ORIGEN_ID");
		sql.append("                  AND TR.DESTINO_ID = CR.DESTINO_ID");
		sql.append("                  AND TF.VIGENCIATARIFA_ID = VT.VIGENCIATARIFA_ID");
		sql.append("                  AND VT.ACTIVO = 1");
		sql.append("                  AND CR.FECCORRIDA BETWEEN VT.FECINICIOVIGENCIA AND");
		sql.append("                      VT.FECFINVIGENCIA) /");
		sql.append("              (SELECT SUM(BO.PRECIOPAGADO + BO.IMPORTEPEDAGIO +");
		sql.append("                          BO.IMPORTESEGURO + BO.IMPORTETAXAEMBARQUE) RECEITA");
		sql.append("                 FROM BOLETO BO");
		sql.append("                WHERE BO.CORRIDA_ID = :CORRIDA_ID");
		sql.append("                  AND BO.FECCORRIDA = :FECCORRIDA");
		sql.append("                  AND BO.ACTIVO = 1");
		sql.append("                  AND BO.INDSTATUSOPERACION = 'F'");
		sql.append("                  AND (BO.MOTIVOCANCELACION_ID IS NULL OR");
		sql.append("                      BO.MOTIVOCANCELACION_ID = 0)),");
		sql.append("              0) VALOR");
		sql.append("   FROM DUAL");

		return sql.toString();
	}

	protected String getSqlIndicadorPax() {
		StringBuilder sql = new StringBuilder();
		sql.append("    SELECT (TAB1.EQUIVALENTE * TAB1.EXTENSAO_LINHA) ");
		sql.append("              VALOR");
		sql.append("        FROM (SELECT TAB.*,");
		sql.append("                     (SELECT SUM((TR.CANTKMREAL / TAB.EXTENSAO_LINHA) *");
		sql.append("                                 COUNT(1))");
		sql.append("                        FROM CORRIDA       CR,");
		sql.append("                             CORRIDA_TRAMO CT,");
		sql.append("                             TRAMO         TR,");
		sql.append("                             BOLETO        BO,");
		sql.append("                             CORRIDA_TRAMO CTD,");
		sql.append("                             CORRIDA_TRAMO CTO");
		sql.append("                       WHERE CR.CORRIDA_ID = :CORRIDA_ID");
		sql.append("                         AND CR.FECCORRIDA = :FECCORRIDA");
		sql.append("                         AND CR.ROLOPERATIVO_ID = :ROLOPERATIVO_ID");
		sql.append("                         AND CT.FECCORRIDA = CR.FECCORRIDA");
		sql.append("                         AND CT.CORRIDA_ID = CR.CORRIDA_ID");
		sql.append("                         AND TR.TRAMO_ID = CT.TRAMO_ID");
		sql.append("                         AND BO.CORRIDA_ID = CT.CORRIDA_ID");
		sql.append("                         AND BO.FECCORRIDA = CT.FECCORRIDA");
		sql.append("                         AND BO.ORIGEN_ID = CTO.ORIGEN_ID");
		sql.append("                         AND BO.DESTINO_ID = CTD.DESTINO_ID");
		sql.append("                         AND CTD.CORRIDA_ID = CT.CORRIDA_ID");
		sql.append("                         AND CTD.FECCORRIDA = CT.FECCORRIDA");
		sql.append("                         AND CTO.CORRIDA_ID = CT.CORRIDA_ID");
		sql.append("                         AND CTO.FECCORRIDA = CT.FECCORRIDA");
		sql.append("                         AND CTO.ACTIVO = 1");
		sql.append("                         AND CTD.ACTIVO = 1");
		sql.append("                         AND CT.NUMSECUENCIA BETWEEN CTO.NUMSECUENCIA AND");
		sql.append("                             CTD.NUMSECUENCIA");
		sql.append("                         AND BO.ACTIVO = 1");
		sql.append("                         AND BO.INDSTATUSOPERACION = 'F'");
		sql.append("                         AND (BO.MOTIVOCANCELACION_ID IS NULL OR");
		sql.append("                             BO.MOTIVOCANCELACION_ID = 0)");
		sql.append("                       GROUP BY CT.TRAMO_ID, TR.CANTKMREAL) EQUIVALENTE");
		sql.append("                FROM (SELECT (SELECT SUM(TR1.CANTKMREAL)");
		sql.append("                                FROM CORRIDA_TRAMO CT1, TRAMO TR1");
		sql.append("                               WHERE CT1.CORRIDA_ID = :CORRIDA_ID");
		sql.append("                                 AND CT1.FECCORRIDA = :FECCORRIDA");
		sql.append("                                 AND TR1.TRAMO_ID = CT1.TRAMO_ID) EXTENSAO_LINHA");
		sql.append("                      ");
		sql.append("                        FROM DUAL) TAB) TAB1");

		return sql.toString();

	}

	protected String getSqlIndicadorIap() {
		StringBuilder sql = new StringBuilder();
		sql.append("      SELECT ROUND(SUM(:CEM * (( EQUIVALENTE * EXTENSAO_LINHA ) / ( :CAPACIDADE * ");
		sql.append("                         EXTENSAO_LINHA ))), ");
		sql.append("                  :MAX_DECIMAL ) VALOR ");
		sql.append("       FROM   (SELECT ROUND(SUM(EQ), :MAX_DECIMAL) EQUIVALENTE, ");
		sql.append("                EL      EXTENSAO_LINHA ");
		sql.append("            FROM   (SELECT ROUND(( BO.NUMKMVIAJE * COUNT(1) / ( EXTENSAO_LINHA.EL ) ");
		sql.append("                             ), :MAX_DECIMAL ) ");
		sql.append("                       EQ, ");
		sql.append("                       EXTENSAO_LINHA.EL ");
		sql.append("                       EL ");
		sql.append("                 FROM   BOLETO BO, ");
		sql.append("                       (SELECT SUM(TR1.CANTKMREAL) EL ");
		sql.append("                        FROM   CORRIDA_TRAMO CT1, ");
		sql.append("                               TRAMO TR1 ");
		sql.append("                        WHERE  CT1.CORRIDA_ID = :CORRIDA_ID ");
		sql.append("                               AND CT1.FECCORRIDA = :FECCORRIDA ");
		sql.append("                               AND TR1.TRAMO_ID = CT1.TRAMO_ID) EXTENSAO_LINHA ");
		sql.append("                 WHERE  BO.CORRIDA_ID = :CORRIDA_ID ");
		sql.append("                       AND BO.FECCORRIDA = :FECCORRIDA ");
		sql.append("                       AND BO.ACTIVO = 1 ");
		sql.append("                       AND ( BO.MOTIVOCANCELACION_ID IS NULL ");
		sql.append("                              OR BO.MOTIVOCANCELACION_ID = 0 ) ");
		sql.append("                 GROUP  BY BO.NUMKMVIAJE, EXTENSAO_LINHA.EL) ");
		sql.append("         GROUP  BY EQ, EL) ");

		return sql.toString();

	}

	protected String getSqlIndicadorReceitaKm() {
		StringBuilder sql = new StringBuilder();
		sql.append(" SELECT (RECEITA / (SELECT SUM(TR1.CANTKMREAL) ");
		sql.append(" FROM CORRIDA_TRAMO CT1 ");
		sql.append("      RIGHT JOIN CORRIDA CR ON  ");
		sql.append("                  (CR.CORRIDA_ID = CT1.CORRIDA_ID AND CR.FECCORRIDA = CT1.FECCORRIDA), ");
		sql.append(" TRAMO TR1 ");
		sql.append(" WHERE CT1.CORRIDA_ID = :CORRIDA_ID ");
		sql.append("   AND CT1.FECCORRIDA = :FECCORRIDA ");
		sql.append("   AND CR.ROLOPERATIVO_ID = :ROLOPERATIVO_ID ");
		sql.append("   AND TR1.TRAMO_ID = CT1.TRAMO_ID)) VALOR ");
		sql.append(" FROM (SELECT SUM(BO.PRECIOPAGADO + BO.IMPORTEPEDAGIO + BO.IMPORTESEGURO + ");
		sql.append("    BO.IMPORTETAXAEMBARQUE) RECEITA ");
		sql.append(" FROM BOLETO BO ");
		sql.append("       RIGHT JOIN CORRIDA CR ON  ");
		sql.append("            (CR.CORRIDA_ID = BO.CORRIDA_ID AND CR.FECCORRIDA = BO.FECCORRIDA) ");
		sql.append(" WHERE BO.CORRIDA_ID = :CORRIDA_ID ");
		sql.append(" AND BO.FECCORRIDA = :FECCORRIDA ");
		sql.append(" AND CR.ROLOPERATIVO_ID = :ROLOPERATIVO_ID ");
		sql.append(" AND BO.ACTIVO = 1 ");
		sql.append(" AND BO.INDSTATUSOPERACION = 'F' ");
		sql.append(" AND (BO.MOTIVOCANCELACION_ID IS NULL OR ");
		sql.append("  BO.MOTIVOCANCELACION_ID = 0)) RC ");

		return sql.toString();

	}

	protected String getSqlIndicadorReceitaViagem() {
		StringBuilder sql = new StringBuilder();

		sql.append("SELECT SUM(BO.PRECIOPAGADO + BO.IMPORTEPEDAGIO + BO.IMPORTESEGURO + ");
		sql.append("    BO.IMPORTETAXAEMBARQUE) VALOR  ");
		sql.append(" FROM BOLETO BO ");
		sql.append("      RIGHT JOIN CORRIDA CR ON  ");
		sql.append("                  (CR.CORRIDA_ID = BO.CORRIDA_ID AND CR.FECCORRIDA = BO.FECCORRIDA) ");
		sql.append(" WHERE BO.CORRIDA_ID = :CORRIDA_ID ");
		sql.append(" AND BO.FECCORRIDA = :FECCORRIDA ");
		sql.append(" AND CR.ROLOPERATIVO_ID = :ROLOPERATIVO_ID ");
		sql.append(" AND BO.ACTIVO = 1 ");
		sql.append(" AND BO.INDSTATUSOPERACION = 'F' ");
		sql.append(" AND (BO.MOTIVOCANCELACION_ID IS NULL OR ");
		sql.append("  BO.MOTIVOCANCELACION_ID = 0) ");

		return sql.toString();

	}

	protected String getSqlIndicadorEquivalentes() {
		/* O calculo do MPE no leva em considerao a quantidade de viagens conforme formula, uma vez que o detalhemtno do relatrio j  por viagem */
		StringBuilder sql = new StringBuilder();
		sql.append(" SELECT SUM((TR.CANTKMREAL/EL.EXTENSAO_LINHA ) * COUNT(1)) VALOR ");
		sql.append("   FROM CORRIDA CR, ");
		sql.append("        CORRIDA_TRAMO CT, ");
		sql.append("        TRAMO TR, ");
		sql.append("        BOLETO BO, ");
		sql.append("        CORRIDA_TRAMO CTD, ");
		sql.append("        CORRIDA_TRAMO CTO, ");
		sql.append("        (SELECT SUM(TR1.CANTKMREAL) EXTENSAO_LINHA ");
		sql.append("           FROM CORRIDA_TRAMO CT1, TRAMO TR1 ");
		sql.append("          WHERE CT1.CORRIDA_ID = :CORRIDA_ID  ");
		sql.append("            AND CT1.FECCORRIDA = :FECCORRIDA  ");
		sql.append("            AND TR1.TRAMO_ID = CT1.TRAMO_ID) EL ");
		sql.append("  WHERE CR.CORRIDA_ID = :CORRIDA_ID ");
		sql.append("    AND CR.FECCORRIDA = :FECCORRIDA   ");
		sql.append("    AND CR.ROLOPERATIVO_ID = :ROLOPERATIVO_ID   ");
		sql.append("    AND CT.FECCORRIDA = CR.FECCORRIDA ");
		sql.append("    AND CT.CORRIDA_ID = CR.CORRIDA_ID ");
		sql.append("    AND TR.TRAMO_ID = CT.TRAMO_ID ");
		sql.append("    AND BO.CORRIDA_ID = CT.CORRIDA_ID ");
		sql.append("    AND BO.FECCORRIDA = CT.FECCORRIDA ");
		sql.append("    AND BO.ORIGEN_ID = CTO.ORIGEN_ID ");
		sql.append("    AND BO.DESTINO_ID = CTD.DESTINO_ID ");
		sql.append("    AND CTD.CORRIDA_ID = CT.CORRIDA_ID ");
		sql.append("    AND CTD.FECCORRIDA = CT.FECCORRIDA ");
		sql.append("    AND CTO.CORRIDA_ID = CT.CORRIDA_ID ");
		sql.append("    AND CTO.FECCORRIDA = CT.FECCORRIDA ");
		sql.append("    AND CTO.ACTIVO = 1 ");
		sql.append("    AND CTD.ACTIVO = 1 ");
		sql.append("    AND CT.NUMSECUENCIA BETWEEN CTO.NUMSECUENCIA AND CTD.NUMSECUENCIA ");
		sql.append("    AND BO.ACTIVO = 1 ");
		sql.append("    AND BO.INDSTATUSOPERACION = 'F' ");
		sql.append("    AND (BO.MOTIVOCANCELACION_ID IS NULL OR BO.MOTIVOCANCELACION_ID = 0) ");
		sql.append("  GROUP BY CT.TRAMO_ID, TR.CANTKMREAL, EL.EXTENSAO_LINHA ");

		return sql.toString();

	}

	protected String getSqlIndicadorAbsoluto() {

		StringBuilder sql = new StringBuilder();

		sql.append(" SELECT SUM(1) AS VALOR");
		sql.append("   FROM BOLETO BO ");
		sql.append("             RIGHT JOIN CORRIDA CR ON  ");
		sql.append("                  (CR.CORRIDA_ID = BO.CORRIDA_ID AND CR.FECCORRIDA = BO.FECCORRIDA) ");
		sql.append("  WHERE BO.INDSTATUSOPERACION = 'F' ");
		sql.append("    AND BO.ACTIVO = 1 ");
		sql.append("    AND (BO.MOTIVOCANCELACION_ID IS NULL OR BO.MOTIVOCANCELACION_ID = 0) ");
		sql.append("    AND BO.CORRIDA_ID = :CORRIDA_ID ");
		sql.append("    AND BO.FECCORRIDA = :FECCORRIDA ");
		sql.append("    AND CR.ROLOPERATIVO_ID = :ROLOPERATIVO_ID ");

		return sql.toString();

	}

	protected BigDecimal getValorIndicador(Date data, Integer corridaId, Integer rolOperativoId, Integer indicador, Integer assentos, String sql) throws SQLException {
		BigDecimal retorno = null;
		try {

			Connection conexao = this.getConexao();

			NamedParameterStatement stmt = new NamedParameterStatement(conexao, sql);

			stmt.setInt("CORRIDA_ID", corridaId);
			stmt.setDate("FECCORRIDA", new java.sql.Date(data.getTime()));

			if (IndicadorRelatorio.fromInt(indicador).equals(IndicadorRelatorio.IAP)) {
				stmt.setInt("CAPACIDADE", assentos);
				stmt.setInt("CEM", CEM);
				stmt.setInt("MAX_DECIMAL", MAX_DECIMAL);
			} else {
				stmt.setInt("ROLOPERATIVO_ID", rolOperativoId);
			}

			ResultSet resultSet = stmt.executeQuery();

			if (resultSet.next())
				retorno = resultSet.getBigDecimal("VALOR");

			resultSet.close();
			stmt.close();
		} catch (Exception e) {
			e.printStackTrace();
		}

		return retorno;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.rjconsultores.ventaboletos.relatorios.utilitarios.Relatorio#processaParametros()
	 */
	@Override
	public void processaParametros() throws Exception {
	}

	private String getSql() {

		StringBuilder sql = new StringBuilder();

		sql.append(" SELECT DISTINCT RT.RUTA_ID, ");
		sql.append("                 CR.CORRIDA_ID, ");
		sql.append("                 RT.DESCRUTA, ");
		sql.append("                 DA.CANTASIENTOS ASSENTOS, ");
		sql.append("                 PO.CVEPARADA||' - '||PD.CVEPARADA SIGLA, ");
		sql.append("                 TO_CHAR(CR.FECHORSALIDAORIGINAL, 'HH24:MI') HORARIO,  ");
		sql.append("                 CR.ROLOPERATIVO_ID, ");
		sql.append("                 CASE ");
		sql.append("                   WHEN CO.ESTADO_ID <> CD.ESTADO_ID THEN ");
		sql.append("                    'S' ");
		sql.append("                   ELSE ");
		sql.append("                    'N' ");
		sql.append("                 END INTERESTADUAL, ");
		sql.append("                 NVL(GR.DESCGRUPO, 'No Definido') GRUPO_LINHA, ");
		sql.append("                 (CASE ");
		sql.append("                   WHEN CR.TIPOSERVICIO_ID = 2 THEN ");
		sql.append("                    'Extra' ");
		sql.append("                   ELSE ");
		sql.append("                    NULL ");
		sql.append("                 END) TIPO_SERVICO ");
		sql.append("  ");
		sql.append("   FROM RUTA             RT, ");
		sql.append("        CORRIDA          CR, ");
		sql.append("        TRAMO            TR, ");
		sql.append("        ROL_OPERATIVO    RO, ");
		sql.append("        DIAGRAMA_AUTOBUS DA, ");
		sql.append("        GRUPO_RUTA       GR, ");
		sql.append("        PARADA           PO, ");
		sql.append("        PARADA           PD, ");
		sql.append("        CIUDAD           CO, ");
		sql.append("        CIUDAD           CD ");
		sql.append("  WHERE RT.RUTA_ID = CR.RUTA_ID ");
		sql.append("    AND CR.EMPRESACORRIDA_ID = NVL(:EMPRESA_ID, CR.EMPRESACORRIDA_ID) ");
		sql.append("    AND CR.ORIGEN_ID = PO.PARADA_ID ");
		sql.append("    AND CR.DESTINO_ID = PD.PARADA_ID ");
		sql.append("    AND PO.CIUDAD_ID = CO.CIUDAD_ID ");
		sql.append("    AND PD.CIUDAD_ID = CD.CIUDAD_ID ");
		sql.append("    AND RT.GRUPORUTA_ID = GR.GRUPORUTA_ID(+) ");
		sql.append("    AND RO.ROLOPERATIVO_ID = CR.ROLOPERATIVO_ID ");
		sql.append("    AND RO.DIAGRAMAAUTOBUS_ID = DA.DIAGRAMAAUTOBUS_ID ");
		sql.append("    AND TR.ORIGEN_ID = CR.ORIGEN_ID ");
		sql.append("    AND TR.DESTINO_ID = CR.DESTINO_ID ");
		sql.append("    AND CR.TIPOSERVICIO_ID = NVL(:TIPOSERVICIO_ID, CR.TIPOSERVICIO_ID)  ");
		sql.append("    AND TO_CHAR(CR.FECCORRIDA, 'MMYYYY') = TO_CHAR(:DATA_MES, 'MMYYYY') ");
		sql.append("    ORDER BY INTERESTADUAL, GRUPO_LINHA, SIGLA");

		return sql.toString();
	}

}
