/**
 * 
 */
package com.rjconsultores.ventaboletos.relatorios.impl;

import java.math.BigDecimal;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.util.ArrayList;
import java.util.List;
import java.util.Locale;
import java.util.Map;

import net.sf.jasperreports.engine.data.JRBeanCollectionDataSource;

import com.rjconsultores.ventaboletos.relatorios.utilitarios.DataSource;
import com.rjconsultores.ventaboletos.relatorios.utilitarios.Relatorio;
import com.rjconsultores.ventaboletos.relatorios.utilitarios.RelatorioAproveitamentoBean;
import com.rjconsultores.ventaboletos.web.utilerias.CurrencyUtil;
import com.rjconsultores.ventaboletos.web.utilerias.NamedParameterStatement;

/**
 * @author Bruno H. G. Gouva <bruno@rjconsultores.com.br>
 * 
 */
public class RelatorioAproveitamento extends Relatorio {

	private List<RelatorioAproveitamentoBean> lsDadosRelatorio;

	public RelatorioAproveitamento(Map<String, Object> parametros, Connection conexao) throws Exception {
		super(parametros, conexao);

		this.setCustomDataSource(new DataSource(this) {
			@Override
			public void initDados() throws Exception {

				Connection conexao = this.relatorio.getConexao();

				Map<String, Object> parametros = this.relatorio.getParametros();

				String sql = getSql();

				PreparedStatement stmt = null;
				ResultSet rset = null;
				stmt = conexao.prepareStatement(sql);

				stmt.setInt(1, Integer.valueOf((String) parametros.get("CORRIDA_ID")));
				stmt.setDate(2, (java.sql.Date) parametros.get("FECCORRIDA"));
				rset = stmt.executeQuery();

				if (rset.next()) {

					Locale locale = new Locale("pt", "BR");

					parametros.put("P_NOMINAL_KM", CurrencyUtil.getCurrencyStrFromStr(rset.getString("NOMINAL_KM"), locale));
					parametros.put("P_TOTAL_ARRECADADO", CurrencyUtil.getCurrencyStrFromStr(rset.getString("TOTAL_ARRECADADO"), locale));
					parametros.put("P_ARRECADO_KM", CurrencyUtil.getCurrencyStrFromStr(rset.getString("ARRECADADO_KM"), locale));
					parametros.put("P_OCUPACAO", CurrencyUtil.getPercentStrFromStr(rset.getString("OCUPACAO"), locale));
					parametros.put("P_OCUPACAO_ARRECADACAO", CurrencyUtil.getPercentStrFromStr(rset.getString("OCUPACAO_ARRECADACAO"), locale));
					parametros.put("P_TOTAL_NOMINAL", CurrencyUtil.getCurrencyStrFromStr(rset.getString("TOTAL_NOMINAL"), locale));
					parametros.put("P_OCUPACAO_S_ARRECADACAO", CurrencyUtil.getPercentStrFromStr(rset.getString("OCUPACAO_S_ARRECADACAO"), locale));
				}

				NamedParameterStatement nps = new NamedParameterStatement(conexao, getSqlDetail());

				nps.setInt("CORRIDA_ID", Integer.valueOf((String) parametros.get("CORRIDA_ID")));
				nps.setDate("FECCORRIDA", (java.sql.Date) parametros.get("FECCORRIDA"));

				rset = nps.executeQuery();

				lsDadosRelatorio = new ArrayList<RelatorioAproveitamentoBean>();

				while (rset.next()) {

					RelatorioAproveitamentoBean aproveitamentoBean = new RelatorioAproveitamentoBean();
					aproveitamentoBean.setCveparada((String) rset.getObject("CVEPARADA"));
					aproveitamentoBean.setAsiento((BigDecimal) rset.getObject("ASIENTO"));
					aproveitamentoBean.setOcupado((BigDecimal) rset.getObject("OCUPADO"));
					aproveitamentoBean.setOcupadoOperacao((BigDecimal) rset.getObject("OCUPADO_OPERACAO"));
					aproveitamentoBean.setNumsecuencia((BigDecimal) rset.getObject("NUMSECUENCIA"));

					lsDadosRelatorio.add(aproveitamentoBean);
				}

				Integer asiento = 1;
				BigDecimal ocupado = null;
				Integer secOcupado = 0;

				for (RelatorioAproveitamentoBean i : lsDadosRelatorio) {
					if (asiento.equals(i.getAsiento().intValue())) {
						if (i.getOcupado() != null) {
							if (secOcupado.equals(0)) {
								secOcupado++;
								ocupado = i.getOcupado();
								i.setOcupado(new BigDecimal(secOcupado));
							} else {
								if (!i.getOcupado().equals(ocupado)) {
									secOcupado++;
									ocupado = i.getOcupado();
									i.setOcupado(new BigDecimal(secOcupado));
								} else {
									i.setOcupado(new BigDecimal(secOcupado));
								}
							}

						}
					} else {
						secOcupado = 0;
						if (i.getOcupado() != null)
							i.setOcupado(BigDecimal.ONE);
					}

					asiento = i.getAsiento().intValue();
				}

				setLsDadosRelatorio(lsDadosRelatorio);

			}
		});

	}

	private String getSqlDetail() {

		StringBuilder sql = new StringBuilder();
		sql.append("SELECT DISTINCT PR.CVEPARADA AS CVEPARADA, ");
		sql.append("                TO_NUMBER(DD.ASIENTO)      AS ASIENTO, ");
		sql.append("                (SELECT BO.BOLETO_ID ");
		sql.append("                 FROM   BOLETO BO, ");
		sql.append("                        CORRIDA_TRAMO CTD, ");
		sql.append("                        CORRIDA_TRAMO CTO ");
		sql.append("                 WHERE  BO.CORRIDA_ID = CR.CORRIDA_ID ");
		sql.append("                        AND BO.FECCORRIDA = CR.FECCORRIDA ");
		sql.append("                        AND BO.NUMASIENTO = DD.ASIENTO ");
		sql.append("                        AND BO.ORIGEN_ID = CTO.ORIGEN_ID ");
		sql.append("                        AND BO.DESTINO_ID = CTD.DESTINO_ID ");
		sql.append("                        AND CTD.CORRIDA_ID = CR.CORRIDA_ID ");
		sql.append("                        AND CTD.FECCORRIDA = CR.FECCORRIDA ");
		sql.append("                        AND CTO.CORRIDA_ID = CR.CORRIDA_ID ");
		sql.append("                        AND CTO.FECCORRIDA = CR.FECCORRIDA ");
		sql.append("                        AND CTO.ACTIVO = 1 ");
		sql.append("                        AND CTD.ACTIVO = 1 ");
		sql.append("                        AND VT.NUMSECUENCIA BETWEEN ");
		sql.append("                            CTO.NUMSECUENCIA AND CTD.NUMSECUENCIA + 1 ");
		sql.append("                        AND BO.ACTIVO = 1 ");
		sql.append("                        AND ( BO.MOTIVOCANCELACION_ID IS NULL ");
		sql.append("                               OR BO.MOTIVOCANCELACION_ID = 0 ) ");
		sql.append("                        AND ROWNUM = 1)    OCUPADO, ");
		sql.append("                (SELECT DECODE(AE.VENDIBLE, 1, 1, ");
		sql.append("                                            0, 2) ");
		sql.append("                 FROM   ASIENTO_EXCLUSIVO AE, ");
		sql.append("                        CORRIDA_TRAMO CTD, ");
		sql.append("                        CORRIDA_TRAMO CTO ");
		sql.append("                 WHERE  AE.CORRIDA_ID = CR.CORRIDA_ID ");
		sql.append("                        AND AE.FECCORRIDA = CR.FECCORRIDA ");
		sql.append("                        AND AE.NUMASIENTO = DD.ASIENTO ");
		sql.append("                        AND AE.PARADA_ID = CTO.ORIGEN_ID ");
		sql.append("                        AND NVL(AE.DESTINO_ID, CR.DESTINO_ID) = CTD.DESTINO_ID ");
		sql.append("                        AND CTD.CORRIDA_ID = CR.CORRIDA_ID ");
		sql.append("                        AND CTD.FECCORRIDA = CR.FECCORRIDA ");
		sql.append("                        AND CTO.CORRIDA_ID = CR.CORRIDA_ID ");
		sql.append("                        AND CTO.FECCORRIDA = CR.FECCORRIDA ");
		sql.append("                        AND CTO.ACTIVO = 1 ");
		sql.append("                        AND CTD.ACTIVO = 1 ");
		sql.append("                        AND VT.NUMSECUENCIA BETWEEN ");
		sql.append("                            CTO.NUMSECUENCIA AND CTD.NUMSECUENCIA + 1 ");
		sql.append("                        AND AE.ACTIVO = 1) OCUPADO_OPERACAO, ");
		sql.append("                VT.NUMSECUENCIA AS NUMSECUENCIA ");
		sql.append("FROM   CORRIDA CR, ");
		sql.append("       (SELECT CT.ORIGEN_ID PARADA_ID, ");
		sql.append("               CT.NUMSECUENCIA ");
		sql.append("        FROM   CORRIDA_TRAMO CT ");
		sql.append("        WHERE  CT.CORRIDA_ID = :CORRIDA_ID ");
		sql.append("               AND TRUNC(CT.FECCORRIDA) = :FECCORRIDA ");
		sql.append("               AND CT.ACTIVO = 1 ");
		sql.append("        UNION ");
		sql.append("        SELECT CT.DESTINO_ID, ");
		sql.append("               CT.NUMSECUENCIA + 1 ");
		sql.append("        FROM   CORRIDA_TRAMO CT ");
		sql.append("        WHERE  CT.CORRIDA_ID = :CORRIDA_ID ");
		sql.append("               AND TRUNC(CT.FECCORRIDA) = :FECCORRIDA ");
		sql.append("               AND CT.ACTIVO = 1 ");
		sql.append("        ORDER  BY 2) VT, ");
		sql.append("       PARADA PR, ");
		sql.append("       ROL_OPERATIVO RO, ");
		sql.append("       DET_DIAGRAMA_AUTOBUS DD ");
		sql.append("WHERE  CR.CORRIDA_ID = :CORRIDA_ID ");
		sql.append("       AND TRUNC(CR.FECCORRIDA) = :FECCORRIDA ");
		sql.append("       AND VT.PARADA_ID = PR.PARADA_ID ");
		sql.append("       AND CR.ROLOPERATIVO_ID = RO.ROLOPERATIVO_ID ");
		sql.append("       AND RO.DIAGRAMAAUTOBUS_ID = DD.DIAGRAMAAUTOBUS_ID ");
		sql.append("       AND DD.VENDIBLE = 1 ");
		sql.append("ORDER  BY TO_NUMBER(DD.ASIENTO), ");
		sql.append("          VT.NUMSECUENCIA ");

		return sql.toString();
	}

	private String getSql() {

		StringBuilder sql = new StringBuilder();

		sql.append("   SELECT round(NVL(TAB.TOTAL_ARRECADADO, 0), 2) TOTAL_ARRECADADO, ");
		sql.append("        round(NVL(TAB.TOTAL_ARRECADADO / TAB.KM_NOMINAL, 0), 2) ARRECADADO_KM, ");
		sql.append("        round(NVL(TAB.TOTAL_NOMINAL, 0), 2) TOTAL_NOMINAL, ");
		sql.append("        NVL(round(TAB.TOTAL_NOMINAL / TAB.KM_NOMINAL, 2), 0) NOMINAL_KM, ");
		sql.append("        ((TAB.TOTAL_OCUPADOS + TAB.TOTAL_OCUPADOS_OPERACAO) / (CASE TAB.TOTAL_ACENTOS WHEN 0 THEN 1 ELSE TAB.TOTAL_ACENTOS END) ) OCUPACAO, ");
		sql.append("        ((TAB.TOTAL_OCUPADOS - TAB.TOTAL_OCUPADOS_GRATUIDADE) / (CASE TAB.TOTAL_ACENTOS WHEN 0 THEN 1 ELSE TAB.TOTAL_ACENTOS END) ) OCUPACAO_ARRECADACAO, ");
		sql.append("        ((TAB.TOTAL_OCUPADOS_GRATUIDADE + TAB.TOTAL_OCUPADOS_OPERACAO) / (CASE TAB.TOTAL_ACENTOS WHEN 0 THEN 1 ELSE TAB.TOTAL_ACENTOS END) ) OCUPACAO_S_ARRECADACAO ");
		sql.append("   FROM (SELECT (SELECT SUM(BO.PRECIOPAGADO) ");
		sql.append("                   FROM BOLETO BO ");
		sql.append("                  WHERE BO.CORRIDA_ID = CR.CORRIDA_ID ");
		sql.append("                    AND BO.FECCORRIDA = CR.FECCORRIDA ");
		sql.append("                    AND BO.ACTIVO = 1 ");
		sql.append("                    AND BO.INDSTATUSOPERACION = 'F' ");
		sql.append("                    AND (BO.MOTIVOCANCELACION_ID IS NULL OR ");
		sql.append("                        BO.MOTIVOCANCELACION_ID = 0)) TOTAL_ARRECADADO, ");
		sql.append("                 ");
		sql.append("                (SELECT SUM(TR.CANTKMREAL) ");
		sql.append("                   FROM CORRIDA_TRAMO    CT, ");
		sql.append("                        TRAMO            TR, ");
		sql.append("                        ROL_OPERATIVO    RO, ");
		sql.append("                        DIAGRAMA_AUTOBUS DA ");
		sql.append("                  WHERE CT.CORRIDA_ID = CR.CORRIDA_ID ");
		sql.append("                    AND CT.FECCORRIDA = CR.FECCORRIDA ");
		sql.append("                    AND RO.ROLOPERATIVO_ID = CR.ROLOPERATIVO_ID ");
		sql.append("                    AND DA.DIAGRAMAAUTOBUS_ID = RO.DIAGRAMAAUTOBUS_ID ");
		sql.append("                    AND TR.TRAMO_ID = CT.TRAMO_ID ");
		sql.append("                    AND TR.ACTIVO = 1 ");
		sql.append("                    AND CT.ACTIVO = 1) KM_NOMINAL, ");
		sql.append("                 ");
		sql.append("                (SELECT SUM((SELECT TR.CANTKMREAL ");
		sql.append("                              FROM BOLETO        BO, ");
		sql.append("                                   CORRIDA_TRAMO CTD, ");
		sql.append("                                   CORRIDA_TRAMO CTO ");
		sql.append("                             WHERE BO.CORRIDA_ID = CR.CORRIDA_ID ");
		sql.append("                               AND BO.FECCORRIDA = CR.FECCORRIDA ");
		sql.append("                               AND BO.NUMASIENTO = DD.ASIENTO ");
		sql.append("                               AND BO.ORIGEN_ID = CTO.ORIGEN_ID ");
		sql.append("                               AND BO.DESTINO_ID = CTD.DESTINO_ID ");
		sql.append("                               AND CTD.CORRIDA_ID = CR.CORRIDA_ID ");
		sql.append("                               AND CTD.FECCORRIDA = CR.FECCORRIDA ");
		sql.append("                               AND CTO.CORRIDA_ID = CR.CORRIDA_ID ");
		sql.append("                               AND CTO.FECCORRIDA = CR.FECCORRIDA ");
		sql.append("                               AND CTO.ACTIVO = 1 ");
		sql.append("                               AND CTD.ACTIVO = 1 ");
		sql.append("                               AND CT.NUMSECUENCIA BETWEEN CTO.NUMSECUENCIA AND ");
		sql.append("                                   CTD.NUMSECUENCIA ");
		sql.append("                               AND BO.ACTIVO = 1 ");
		sql.append("                               AND BO.INDSTATUSOPERACION = 'F' ");
		sql.append("                               AND (BO.MOTIVOCANCELACION_ID IS NULL OR ");
		sql.append("                                   BO.MOTIVOCANCELACION_ID = 0))) ");
		sql.append("                   FROM CORRIDA_TRAMO        CT, ");
		sql.append("                        TRAMO                TR, ");
		sql.append("                        ROL_OPERATIVO        RO, ");
		sql.append("                        DET_DIAGRAMA_AUTOBUS DD ");
		sql.append("                  WHERE CR.FECCORRIDA = CT.FECCORRIDA ");
		sql.append("                    AND CR.CORRIDA_ID = CT.CORRIDA_ID ");
		sql.append("                    AND CT.TRAMO_ID = TR.TRAMO_ID ");
		sql.append("                    AND CR.ROLOPERATIVO_ID = RO.ROLOPERATIVO_ID ");
		sql.append("                    AND RO.DIAGRAMAAUTOBUS_ID = DD.DIAGRAMAAUTOBUS_ID ");
		sql.append("                    AND DD.VENDIBLE = 1 ");
		sql.append("                    AND CT.ACTIVO = 1) KM_OCUPADA, ");
		sql.append("                 ");
		sql.append("                (SELECT SUM(TF.PRECIO) ");
		sql.append("                   FROM TRAMO                TR, ");
		sql.append("                        ROL_OPERATIVO        RO, ");
		sql.append("                        DET_DIAGRAMA_AUTOBUS DD, ");
		sql.append("                        TARIFA               TF, ");
		sql.append("                        VIGENCIA_TARIFA      VT ");
		sql.append("                  WHERE RO.ROLOPERATIVO_ID = CR.ROLOPERATIVO_ID ");
		sql.append("                    AND RO.DIAGRAMAAUTOBUS_ID = DD.DIAGRAMAAUTOBUS_ID ");
		sql.append("                    AND TF.CLASESERVICIO_ID = CR.CLASESERVICIO_ID ");
		sql.append("                    AND DD.VENDIBLE = 1 ");
		sql.append("                    AND DD.ACTIVO = 1 ");
		sql.append("                    AND TR.ORIGEN_ID = CR.ORIGEN_ID ");
		sql.append("                    AND TR.DESTINO_ID = CR.DESTINO_ID ");
		sql.append("                    AND TF.MARCA_ID = CR.MARCA_ID ");
		sql.append("                    AND TF.RUTA_ID = CR.RUTA_ID ");
		sql.append("                    AND TF.TRAMO_ID = TR.TRAMO_ID ");
		sql.append("                    AND TF.STATUSTARIFA = 'A' ");
		sql.append("                    AND TF.ACTIVO = 1 ");
		sql.append("                    AND TF.VIGENCIATARIFA_ID = VT.VIGENCIATARIFA_ID ");
		sql.append("                    AND VT.ACTIVO = 1 ");
		sql.append("                    AND CR.FECCORRIDA BETWEEN VT.FECINICIOVIGENCIA AND ");
		sql.append("                        VT.FECFINVIGENCIA) TOTAL_NOMINAL, ");
		sql.append("                 ");
		sql.append("                (SELECT COUNT(1) ");
		sql.append("                    FROM CORRIDA_TRAMO        CT, ");
		sql.append("                         TRAMO                TR, ");
		sql.append("                         ROL_OPERATIVO        RO, ");
		sql.append("                         DET_DIAGRAMA_AUTOBUS DD ");
		sql.append("                   WHERE ");
		sql.append("                   ");
		sql.append("                   CR.FECCORRIDA = CT.FECCORRIDA ");
		sql.append("                AND CR.CORRIDA_ID = CT.CORRIDA_ID ");
		sql.append("                AND CT.TRAMO_ID = TR.TRAMO_ID ");
		sql.append("                AND CR.ROLOPERATIVO_ID = RO.ROLOPERATIVO_ID ");
		sql.append("                AND RO.DIAGRAMAAUTOBUS_ID = DD.DIAGRAMAAUTOBUS_ID ");
		sql.append("                AND DD.VENDIBLE = 1 ");
		sql.append("                AND DD.ACTIVO = 1 ");
		sql.append("                AND CT.ACTIVO = 1) TOTAL_ACENTOS, ");
		sql.append("                 ");
		sql.append("                (SELECT COUNT((SELECT 1 ");
		sql.append("                                FROM BOLETO        BO, ");
		sql.append("                                     CORRIDA_TRAMO CTD, ");
		sql.append("                                     CORRIDA_TRAMO CTO ");
		sql.append("                               WHERE BO.CORRIDA_ID = CR.CORRIDA_ID ");
		sql.append("                                 AND BO.FECCORRIDA = CR.FECCORRIDA ");
		sql.append("                                 AND BO.NUMASIENTO = DD.ASIENTO ");
		sql.append("                                 AND BO.ORIGEN_ID = CTO.ORIGEN_ID ");
		sql.append("                                 AND BO.DESTINO_ID = CTD.DESTINO_ID ");
		sql.append("                                 AND CTD.CORRIDA_ID = CR.CORRIDA_ID ");
		sql.append("                                 AND CTD.FECCORRIDA = CR.FECCORRIDA ");
		sql.append("                                 AND CTO.CORRIDA_ID = CR.CORRIDA_ID ");
		sql.append("                                 AND CTO.FECCORRIDA = CR.FECCORRIDA ");
		sql.append("                                 AND CTO.ACTIVO = 1 ");
		sql.append("                                 AND CTD.ACTIVO = 1 ");
		sql.append("                                 AND CT.NUMSECUENCIA BETWEEN CTO.NUMSECUENCIA AND ");
		sql.append("                                     CTD.NUMSECUENCIA ");
		sql.append("                                 AND BO.ACTIVO = 1 ");
		sql.append("                                 AND BO.INDSTATUSOPERACION = 'F' ");
		sql.append("                                 AND (BO.MOTIVOCANCELACION_ID IS NULL OR ");
		sql.append("                                     BO.MOTIVOCANCELACION_ID = 0))) ");
		sql.append("                   FROM CORRIDA_TRAMO        CT, ");
		sql.append("                        TRAMO                TR, ");
		sql.append("                        ROL_OPERATIVO        RO, ");
		sql.append("                        DET_DIAGRAMA_AUTOBUS DD ");
		sql.append("                  WHERE CR.FECCORRIDA = CT.FECCORRIDA ");
		sql.append("                    AND CR.CORRIDA_ID = CT.CORRIDA_ID ");
		sql.append("                    AND CT.TRAMO_ID = TR.TRAMO_ID ");
		sql.append("                    AND CR.ROLOPERATIVO_ID = RO.ROLOPERATIVO_ID ");
		sql.append("                    AND RO.DIAGRAMAAUTOBUS_ID = DD.DIAGRAMAAUTOBUS_ID ");
		sql.append("                    AND DD.VENDIBLE = 1 ");
		sql.append("                    AND DD.ACTIVO = 1 ");
		sql.append("                    AND CT.ACTIVO = 1) TOTAL_OCUPADOS, ");
		sql.append("                (SELECT COUNT((SELECT 1 ");
		sql.append("                                FROM BOLETO        BO, ");
		sql.append("                                     CORRIDA_TRAMO CTD, ");
		sql.append("                                     CORRIDA_TRAMO CTO ");
		sql.append("                               WHERE BO.CORRIDA_ID = CR.CORRIDA_ID ");
		sql.append("                                 AND BO.FECCORRIDA = CR.FECCORRIDA ");
		sql.append("                                 AND BO.NUMASIENTO = DD.ASIENTO ");
		sql.append("                                 AND BO.ORIGEN_ID = CTO.ORIGEN_ID ");
		sql.append("                                 AND BO.DESTINO_ID = CTD.DESTINO_ID ");
		sql.append("                                 AND CTD.CORRIDA_ID = CR.CORRIDA_ID ");
		sql.append("                                 AND CTD.FECCORRIDA = CR.FECCORRIDA ");
		sql.append("                                 AND CTO.CORRIDA_ID = CR.CORRIDA_ID ");
		sql.append("                                 AND CTO.FECCORRIDA = CR.FECCORRIDA ");
		sql.append("                                 AND BO.PRECIOBASE = 0 ");
		sql.append("                                 AND CTO.ACTIVO = 1 ");
		sql.append("                                 AND CTD.ACTIVO = 1 ");
		sql.append("                                 AND CT.NUMSECUENCIA BETWEEN CTO.NUMSECUENCIA AND ");
		sql.append("                                     CTD.NUMSECUENCIA ");
		sql.append("                                 AND BO.ACTIVO = 1 ");
		sql.append("                                 AND BO.INDSTATUSOPERACION = 'F' ");
		sql.append("                                 AND (BO.MOTIVOCANCELACION_ID IS NULL OR ");
		sql.append("                                     BO.MOTIVOCANCELACION_ID = 0))) ");
		sql.append("                   FROM CORRIDA_TRAMO        CT, ");
		sql.append("                        TRAMO                TR, ");
		sql.append("                        ROL_OPERATIVO        RO, ");
		sql.append("                        DET_DIAGRAMA_AUTOBUS DD ");
		sql.append("                  WHERE CR.FECCORRIDA = CT.FECCORRIDA ");
		sql.append("                    AND CR.CORRIDA_ID = CT.CORRIDA_ID ");
		sql.append("                    AND CT.TRAMO_ID = TR.TRAMO_ID ");
		sql.append("                    AND CR.ROLOPERATIVO_ID = RO.ROLOPERATIVO_ID ");
		sql.append("                    AND RO.DIAGRAMAAUTOBUS_ID = DD.DIAGRAMAAUTOBUS_ID ");
		sql.append("                    AND DD.VENDIBLE = 1 ");
		sql.append("                    AND DD.ACTIVO = 1 ");
		sql.append("                    AND CT.ACTIVO = 1) TOTAL_OCUPADOS_GRATUIDADE, ");
		sql.append("                 ");
		sql.append("                (SELECT COUNT((SELECT 1 ");
		sql.append("                                FROM ASIENTO_EXCLUSIVO AE, ");
		sql.append("                                     CORRIDA_TRAMO     CTD, ");
		sql.append("                                     CORRIDA_TRAMO     CTO ");
		sql.append("                               WHERE AE.CORRIDA_ID = CR.CORRIDA_ID ");
		sql.append("                                 AND AE.FECCORRIDA = CR.FECCORRIDA ");
		sql.append("                                 AND AE.NUMASIENTO = DD.ASIENTO ");
		sql.append("                                 AND AE.PARADA_ID = CTO.ORIGEN_ID ");
		sql.append("                                 AND NVL(AE.DESTINO_ID, CR.DESTINO_ID) = ");
		sql.append("                                     CTD.DESTINO_ID ");
		sql.append("                                 AND CTD.CORRIDA_ID = CR.CORRIDA_ID ");
		sql.append("                                 AND CTD.FECCORRIDA = CR.FECCORRIDA ");
		sql.append("                                 AND CTO.CORRIDA_ID = CR.CORRIDA_ID ");
		sql.append("                                 AND CTO.FECCORRIDA = CR.FECCORRIDA ");
		sql.append("                                 AND CTO.ACTIVO = 1 ");
		sql.append("                                 AND CTD.ACTIVO = 1 ");
		sql.append("                                 AND CT.NUMSECUENCIA BETWEEN CTO.NUMSECUENCIA AND ");
		sql.append("                                     CTD.NUMSECUENCIA ");
		sql.append("                                 AND AE.ACTIVO = 1)) ");
		sql.append("                   FROM CORRIDA_TRAMO        CT, ");
		sql.append("                        TRAMO                TR, ");
		sql.append("                        ROL_OPERATIVO        RO, ");
		sql.append("                        DET_DIAGRAMA_AUTOBUS DD ");
		sql.append("                  WHERE CR.FECCORRIDA = CT.FECCORRIDA ");
		sql.append("                    AND CR.CORRIDA_ID = CT.CORRIDA_ID ");
		sql.append("                    AND CT.TRAMO_ID = TR.TRAMO_ID ");
		sql.append("                    AND CR.ROLOPERATIVO_ID = RO.ROLOPERATIVO_ID ");
		sql.append("                    AND RO.DIAGRAMAAUTOBUS_ID = DD.DIAGRAMAAUTOBUS_ID ");
		sql.append("                    AND DD.VENDIBLE = 1 ");
		sql.append("                    AND DD.ACTIVO = 1 ");
		sql.append("                    AND CT.ACTIVO = 1) TOTAL_OCUPADOS_OPERACAO ");
		sql.append("           FROM CORRIDA CR ");
		sql.append("          WHERE CR.CORRIDA_ID = ? ");
		sql.append("            AND CR.FECCORRIDA = ? ) TAB ");

		return sql.toString();
	}

	public List<RelatorioAproveitamentoBean> getLsDadosRelatorio() {
		return lsDadosRelatorio;
	}

	public void setLsDadosRelatorio(List<RelatorioAproveitamentoBean> lsDadosRelatorio) {
		this.setCollectionDataSource(new JRBeanCollectionDataSource(lsDadosRelatorio));
		this.lsDadosRelatorio = lsDadosRelatorio;
	}

	@Override
	protected void processaParametros() throws Exception {
	}

}
