package com.rjconsultores.ventaboletos.relatorios.impl;

import java.sql.Connection;
import java.sql.ResultSet;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import net.sf.jasperreports.engine.data.JRBeanCollectionDataSource;

import com.rjconsultores.ventaboletos.relatorios.utilitarios.DataSource;
import com.rjconsultores.ventaboletos.relatorios.utilitarios.Relatorio;
import com.rjconsultores.ventaboletos.relatorios.utilitarios.RelatorioEmpresaCorridaBean;
import com.rjconsultores.ventaboletos.web.utilerias.NamedParameterStatement;

public class RelatorioEmpresaCorrida extends Relatorio {

	private List<RelatorioEmpresaCorridaBean> lsDadosRelatorio;
	
	public RelatorioEmpresaCorrida(Map<String, Object> parametros, Connection conexao)  throws Exception {
		super(parametros, conexao);
		
		this.setCustomDataSource(new DataSource(this) {
			@Override
			public void initDados() throws Exception {

				Connection conexao = this.relatorio.getConexao();

				Map<String, Object> parametros = this.relatorio.getParametros();

				String fecInicio = parametros.get("fecInicio").toString() + " 00:00:00";
				String fecFinal = parametros.get("fecFinal").toString() + " 23:59:59";
				String empresa = parametros.get("empresa") != null ? parametros.get("empresa").toString() : "";
				
				String sql = getSql(empresa);
		
				System.out.println(sql);
		
				NamedParameterStatement stmt = new NamedParameterStatement(conexao, sql);
				ResultSet rset = null;
		
				SimpleDateFormat sdf = new SimpleDateFormat("dd/MM/yyyy hh:mm:ss");
				stmt.setTimestamp("fecInicio", new java.sql.Timestamp(sdf.parse(fecInicio).getTime()));
				stmt.setTimestamp("fecFinal", new java.sql.Timestamp(sdf.parse(fecFinal).getTime()));
				if (empresa != null && !empresa.equals("")){
					stmt.setInt("empresa_id", Integer.parseInt(empresa));
				}
				rset = stmt.executeQuery();
		
				lsDadosRelatorio = new ArrayList<RelatorioEmpresaCorridaBean>();
				
				while (rset.next()) {
					RelatorioEmpresaCorridaBean empresaCorridaBean = new RelatorioEmpresaCorridaBean();
					empresaCorridaBean.setAbonos(rset.getBigDecimal("abonos"));
					empresaCorridaBean.setBoletos(rset.getBigDecimal("boletos"));
					empresaCorridaBean.setEmpresa(rset.getString("nombempresa"));
					empresaCorridaBean.setKmOnibusArrendados(rset.getInt("kmOnibusArrendados"));
					empresaCorridaBean.setKmOnibusProprio(rset.getInt("kmOnibusProprio"));
					empresaCorridaBean.setKmTotal(rset.getInt("kmTotal"));
					empresaCorridaBean.setLinea(rset.getString("linea"));
					empresaCorridaBean.setPasajerosTransportados(rset.getInt("passageirosTransp"));
					
					lsDadosRelatorio.add(empresaCorridaBean);
				}
				
				if (lsDadosRelatorio.size() > 0) {

					setLsDadosRelatorio(lsDadosRelatorio);
				}
			}
		});
	}
	
	public void setLsDadosRelatorio(List<RelatorioEmpresaCorridaBean> lsDadosRelatorio) {
		this.setCollectionDataSource(new JRBeanCollectionDataSource(lsDadosRelatorio));
		this.lsDadosRelatorio = lsDadosRelatorio;
	}

	@Override
	protected void processaParametros() throws Exception {
	}
	
	private String getSql(String empresa) {
		StringBuffer sql = new StringBuffer();
		sql.append("SELECT DISTINCT e.empresa_id, ");
		sql.append("                r.ruta_id, ");
		sql.append("                r.descruta                          AS linea, ");
		sql.append("                nombempresa, ");
		sql.append("                COALESCE(Sum(CASE ");
		sql.append("                               WHEN ((b.motivocancelacion_id is null) AND (cat.grupocategoria_id is null OR cat.grupocategoria_id <> 4)) THEN ");
		sql.append("                               b.preciopagado ");
		sql.append("                               ELSE 0 ");
		sql.append("                             END), 0)               boletos, ");
		sql.append("                COALESCE(Sum(CASE ");
		sql.append("                               WHEN b.motivocancelacion_id is null AND cat.grupocategoria_id = 4 THEN ");
		sql.append("                               b.preciopagado ");
		sql.append("                               ELSE 0 ");
		sql.append("                             END), 0)               abonos, ");
		sql.append("                COALESCE(SUM(CASE WHEN b.motivocancelacion_id is null THEN 1 ELSE 0 END), 0) passageirosTransp, ");
		sql.append("                COALESCE(km2.kmtotal, 0)            kmtotal, ");
		sql.append("                COALESCE(km2.kmonibusproprio, 0)    kmOnibusProprio, ");
		sql.append("                COALESCE(km2.kmonibusarrendados, 0) kmOnibusArrendados ");
		sql.append("FROM   boleto b ");
		sql.append("       INNER JOIN categoria cat on cat.categoria_id = b.categoria_id ");
		sql.append("       INNER JOIN marca m ");
		sql.append("               ON m.marca_id = b.marca_id ");
		sql.append("       INNER JOIN empresa e ");
		sql.append("               ON e.empresa_id = m.empresa_id ");
		sql.append("       INNER JOIN ruta r ");
		sql.append("               ON r.ruta_id = b.ruta_id ");
		sql.append("       LEFT JOIN (SELECT c1.ruta_id                      ruta_id, ");
		sql.append("                         c1.EMPRESACORRIDA_ID empresa_id, ");
		sql.append("                         COALESCE(Sum(t1.cantkmreal), 0) kmtotal, ");
		sql.append("                         COALESCE(Sum(CASE ");
		sql.append("                                        WHEN a.empresa_id = c1.empresacorrida_id ");
		sql.append("                                      THEN ");
		sql.append("                                        t1.cantkmreal ");
		sql.append("                                        ELSE 0 ");
		sql.append("                                      END), 0)           kmOnibusProprio, ");
		sql.append("                         COALESCE(Sum(CASE ");
		sql.append("                                        WHEN a.empresa_id <> ");
		sql.append("                                             c1.empresacorrida_id THEN ");
		sql.append("                                        t1.cantkmreal ");
		sql.append("                                        ELSE 0 ");
		sql.append("                                      END), 0)           kmOnibusArrendados ");
		sql.append("                  FROM   tramo t1 ");
		sql.append("                         INNER JOIN corrida_tramo ct ");
		sql.append("                                 ON ct.tramo_id = t1.tramo_id ");
		sql.append("                         INNER JOIN corrida c1 ");
		sql.append("                                 ON ct.corrida_id = c1.corrida_id ");
		sql.append("                                    AND ct.feccorrida = c1.feccorrida ");
		sql.append("                         INNER JOIN autobus a ");
		sql.append("                                 ON ct.autobus_id = a.autobus_id ");
		sql.append("                  WHERE  c1.feccorrida BETWEEN :fecInicio AND :fecFinal ");
		sql.append("                         AND c1.empresacorrida_id = :empresa_id ");
		sql.append("                         AND c1.activo <> 0 AND ct.activo <> 0 ");
		sql.append("                         AND a.activo <> 0 AND t1.activo <> 0 ");
		sql.append("                  GROUP  BY c1.ruta_id, ");
		sql.append("                            c1.EMPRESACORRIDA_ID) km2 ");
		sql.append("              ON ( km2.ruta_id = r.ruta_id ");
		sql.append("                   AND km2.empresa_id = e.empresa_id ) ");
		sql.append("WHERE  b.fechorventa BETWEEN :fecInicio AND :fecFinal ");
		if (empresa != null && !empresa.isEmpty()){
			sql.append("       AND e.empresa_id = :empresa_id ");
		}
		sql.append("GROUP  BY e.empresa_id, ");
		sql.append("          r.ruta_id, ");
		sql.append("          r.descruta, ");
		sql.append("          e.nombempresa, ");
		sql.append("          km2.kmtotal, ");
		sql.append("          km2.kmonibusproprio, ");
		sql.append("          km2.kmonibusarrendados ");
		sql.append("ORDER  BY nombempresa, ");
		sql.append("          linea ");
		
		return sql.toString();
	}
}
