/**
 * 
 */
package com.rjconsultores.ventaboletos.relatorios.impl;

import java.math.BigDecimal;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.Timestamp;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import com.rjconsultores.ventaboletos.relatorios.negocio.CalculoImposto;
import com.rjconsultores.ventaboletos.relatorios.utilitarios.ArrayDataSource;
import com.rjconsultores.ventaboletos.relatorios.utilitarios.ExceptionConfiguracao;
import com.rjconsultores.ventaboletos.relatorios.utilitarios.Relatorio;
import com.rjconsultores.ventaboletos.utilerias.DateUtil;
import com.rjconsultores.ventaboletos.web.utilerias.NamedParameterStatement;

/**
 * @author Bruno H. G. Gouva <bruno@rjconsultores.com.br>
 * 
 */
public class RelatorioTaxasLinha extends Relatorio {

	/**
	 * @param parametros
	 * @param conexao
	 * @throws Exception
	 */
	public RelatorioTaxasLinha(Map<String, Object> parametros, Connection conexao) throws Exception {
		super(parametros, conexao);
		CalculoImposto.limpaCache();

		this.setCustomDataSource(new ArrayDataSource(this) {

			public void initDados() throws Exception {
				Connection conexao = this.relatorio.getConexao();
				Map<String, Object> parametros = this.relatorio.getParametros();
				String puntosVentaIds = (String) parametros.get("NUMPUNTOVENTA");

				String sql = getSql(puntosVentaIds);

				NamedParameterStatement stmt = new NamedParameterStatement(conexao, sql);

				stmt.setInt("TIPO_DATA", (Integer) parametros.get("TIPO_DATA"));
				stmt.setTimestamp("DATA_INICIAL", new Timestamp(DateUtil.inicioFecha((Date) parametros.get("DATA_INICIAL")).getTime()));
				stmt.setTimestamp("DATA_FINAL", new Timestamp(DateUtil.fimFecha((Date) parametros.get("DATA_FINAL")).getTime()));

				if (parametros.get("EMPRESA_ID") != null)
					stmt.setInt("EMPRESA_ID", (Integer) parametros.get("EMPRESA_ID"));
				else
					stmt.setNull("EMPRESA_ID", java.sql.Types.INTEGER);

				if (parametros.get("ORIGEN_ID") != null)
					stmt.setInt("ORIGEN_ID", (Integer) parametros.get("ORIGEN_ID"));
				else
					stmt.setNull("ORIGEN_ID", java.sql.Types.INTEGER);

				if (parametros.get("DESTINO_ID") != null)
					stmt.setInt("DESTINO_ID", (Integer) parametros.get("DESTINO_ID"));
				else
					stmt.setNull("DESTINO_ID", java.sql.Types.INTEGER);

				ResultSet rset = stmt.executeQuery();

				while (rset.next()) {

					Map<String, Object> dataResult = new HashMap<String, Object>();

					dataResult.put("PREFIXO", rset.getString("PREFIXO"));
					dataResult.put("CIDADE_ORIGEM", rset.getString("CIDADE_ORIGEM"));
					dataResult.put("CIDADE_DESTINO", rset.getString("CIDADE_DESTINO"));
					dataResult.put("IMPORTETAXAEMBARQUE", rset.getBigDecimal("IMPORTETAXAEMBARQUE"));
					dataResult.put("IMPORTEPEDAGIO", rset.getBigDecimal("IMPORTEPEDAGIO"));
					dataResult.put("IMPORTESEGURO", rset.getBigDecimal("IMPORTESEGURO"));
					dataResult.put("NOMBEMPRESA", rset.getString("NOMBEMPRESA"));
					dataResult.put("NOMBPUNTOVENTA", rset.getString("NOMBPUNTOVENTA"));
					dataResult.put("EMBARQUE_VENDIDOS", rset.getBigDecimal("EMBARQUE_VENDIDOS"));
					dataResult.put("PEDAGIO_VENDIDOS", rset.getBigDecimal("PEDAGIO_VENDIDOS"));
					dataResult.put("SEGURO_VENDIDOS", rset.getBigDecimal("SEGURO_VENDIDOS"));
					dataResult.put("ORIGEM_ESTADO_ID", rset.getInt("ORIGEM_ESTADO_ID"));
					dataResult.put("DESTINO_ESTADO_ID", rset.getInt("DESTINO_ESTADO_ID"));
					dataResult.put("INTERESTADUAL", rset.getString("INTERESTADUAL"));
					dataResult.put("TOTAL_EMBARQUE", rset.getBigDecimal("TOTAL_EMBARQUE"));
					dataResult.put("TOTAL_PEDAGIO", rset.getBigDecimal("TOTAL_PEDAGIO"));
					dataResult.put("TOTAL_SEGURO", rset.getBigDecimal("TOTAL_SEGURO"));

					this.dados.add(dataResult);
				}

				this.resultSet = rset;

			}

			public Object valueCustomFields(String fieldName) throws Exception {

				// Se no for pra subtrair o ICMS, realiza o calculo e adiciona o mesmo
				if ((Boolean) this.relatorio.getParametros().get("B_SUBTRAIR_ICMS")) {
					BigDecimal valorIcms = BigDecimal.ZERO;
					String indInterestadual = this.resultSet.getString("INTERESTADUAL");
					Integer idEstado = this.resultSet.getInt("ORIGEM_ESTADO_ID");

					try {
						if (fieldName.equals("IMPORTETAXAEMBARQUE") && this.resultSet.getBigDecimal("IMPORTETAXAEMBARQUE") != null && !this.resultSet.getBigDecimal("IMPORTETAXAEMBARQUE").equals(BigDecimal.ZERO)) {
							valorIcms = CalculoImposto.getValorImposto(this.relatorio.getConexao(), idEstado, this.resultSet.getInt("EMPRESA_ID"), indInterestadual, BigDecimal.ZERO, BigDecimal.ZERO, this.resultSet.getBigDecimal("IMPORTETAXAEMBARQUE"), BigDecimal.ZERO);
							return this.resultSet.getBigDecimal("IMPORTETAXAEMBARQUE").subtract(valorIcms);
						}
						else if (fieldName.equals("IMPORTEPEDAGIO") && this.resultSet.getBigDecimal("IMPORTEPEDAGIO") != null && !this.resultSet.getBigDecimal("IMPORTEPEDAGIO").equals(BigDecimal.ZERO)) {
							valorIcms = CalculoImposto.getValorImposto(this.relatorio.getConexao(), idEstado, this.resultSet.getInt("EMPRESA_ID"), indInterestadual, BigDecimal.ZERO, BigDecimal.ZERO, BigDecimal.ZERO, this.resultSet.getBigDecimal("IMPORTEPEDAGIO"));
							return this.resultSet.getBigDecimal("IMPORTEPEDAGIO").subtract(valorIcms);
						}
						else if (fieldName.equals("IMPORTESEGURO") && this.resultSet.getBigDecimal("IMPORTESEGURO") != null && !this.resultSet.getBigDecimal("IMPORTESEGURO").equals(BigDecimal.ZERO)) {
							valorIcms = CalculoImposto.getValorImposto(this.relatorio.getConexao(), idEstado, this.resultSet.getInt("EMPRESA_ID"), indInterestadual, BigDecimal.ZERO, this.resultSet.getBigDecimal("IMPORTESEGURO"), BigDecimal.ZERO, BigDecimal.ZERO);
							return this.resultSet.getBigDecimal("IMPORTESEGURO").subtract(valorIcms);
						}
						else if (fieldName.equals("TOTAL_EMBARQUE") && !this.resultSet.getBigDecimal("TOTAL_EMBARQUE").equals(BigDecimal.ZERO)) {
							valorIcms = CalculoImposto.getValorImposto(this.relatorio.getConexao(), idEstado, this.resultSet.getInt("EMPRESA_ID"), indInterestadual, BigDecimal.ZERO, this.resultSet.getBigDecimal("TOTAL_EMBARQUE"), BigDecimal.ZERO, BigDecimal.ZERO);
							return this.resultSet.getBigDecimal("TOTAL_EMBARQUE").subtract(valorIcms);
						}
						else if (fieldName.equals("TOTAL_PEDAGIO") && !this.resultSet.getBigDecimal("TOTAL_PEDAGIO").equals(BigDecimal.ZERO)) {
							valorIcms = CalculoImposto.getValorImposto(this.relatorio.getConexao(), idEstado, this.resultSet.getInt("EMPRESA_ID"), indInterestadual, BigDecimal.ZERO, BigDecimal.ZERO, BigDecimal.ZERO, this.resultSet.getBigDecimal("TOTAL_PEDAGIO"));
							return this.resultSet.getBigDecimal("TOTAL_PEDAGIO").subtract(valorIcms);
						}
						else if (fieldName.equals("TOTAL_SEGURO") && !this.resultSet.getBigDecimal("TOTAL_SEGURO").equals(BigDecimal.ZERO)) {
							valorIcms = CalculoImposto.getValorImposto(this.relatorio.getConexao(), idEstado, this.resultSet.getInt("EMPRESA_ID"), indInterestadual, BigDecimal.ZERO, this.resultSet.getBigDecimal("TOTAL_SEGURO"), BigDecimal.ZERO, BigDecimal.ZERO);
							return this.resultSet.getBigDecimal("TOTAL_SEGURO").subtract(valorIcms);
						}
					} catch (Exception e) {
						if (e instanceof ExceptionConfiguracao)
							this.relatorio.addInfoMsg(e.getMessage());
						else
							throw e;
					}
				}

				return null;
			}
		});
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.rjconsultores.ventaboletos.relatorios.utilitarios.Relatorio#processaParametros()
	 */
	@Override
	protected void processaParametros() throws Exception {
	}

	public HashMap<String, Object> getConfigImposto(Integer idEstado, Integer puntoVentaId) throws Exception {
		HashMap<String, Object> cacheConfig = null;

		try {
			cacheConfig = (HashMap<String, Object>) CalculoImposto.getConfigImpostoByEstadoOrigem(this.getConexao(), idEstado, puntoVentaId);
		} catch (Exception e) {
			if (e instanceof ExceptionConfiguracao)
				this.addInfoMsg(e.getMessage());
			else
				throw e;
		}

		return cacheConfig;
	}

	private String getSql(String puntosVentaIds) {
		StringBuilder sql = new StringBuilder();

		sql.append(" SELECT RT.PREFIXO, ");
		sql.append("        PO.DESCPARADA CIDADE_ORIGEM, ");
		sql.append("        PD.DESCPARADA CIDADE_DESTINO, ");
		sql.append("        CJ.IMPORTETAXAEMBARQUE, ");
		sql.append("        CJ.IMPORTEPEDAGIO, ");
		sql.append("        CJ.IMPORTESEGURO, ");
		sql.append("        EM.NOMBEMPRESA, ");
		sql.append("        EM.EMPRESA_ID, ");
		sql.append("        CJ.PUNTOVENTA_ID, ");
		sql.append("        PV.NOMBPUNTOVENTA, ");
		sql.append("        SUM(CASE WHEN CJ.IMPORTETAXAEMBARQUE>0 THEN 1 ELSE 0 END) EMBARQUE_VENDIDOS, ");
		sql.append("        SUM(CASE WHEN CJ.IMPORTEPEDAGIO>0 THEN 1 ELSE 0 END) PEDAGIO_VENDIDOS, ");
		sql.append("        SUM(CASE WHEN CJ.IMPORTESEGURO>0 THEN 1 ELSE 0 END) SEGURO_VENDIDOS, ");
		sql.append("        CO.ESTADO_ID ORIGEM_ESTADO_ID, ");
		sql.append("        CD.ESTADO_ID DESTINO_ESTADO_ID, ");
		sql.append("        CASE ");
		sql.append("          WHEN CO.ESTADO_ID <> CD.ESTADO_ID THEN ");
		sql.append("           'S' ");
		sql.append("          ELSE ");
		sql.append("           'N' ");
		sql.append("        END INTERESTADUAL, ");
		sql.append("        SUM(CJ.IMPORTETAXAEMBARQUE) TOTAL_EMBARQUE, ");
		sql.append("        SUM(CJ.IMPORTEPEDAGIO) TOTAL_PEDAGIO, ");
		sql.append("        SUM(CJ.IMPORTESEGURO) TOTAL_SEGURO ");
		sql.append("   FROM CORRIDA     CR, ");
		sql.append("        RUTA        RT, ");
		sql.append("        PUNTO_VENTA PV, ");
		sql.append("        CAJA        CJ, ");
		sql.append("        PARADA      PO, ");
		sql.append("        PARADA      PD, ");
		sql.append("        CIUDAD      CO, ");
		sql.append("        CIUDAD      CD, ");
		sql.append("        EMPRESA     EM ");
		sql.append("  WHERE CR.RUTA_ID = RT.RUTA_ID ");
		sql.append("   AND (CJ.IMPORTETAXAEMBARQUE>0 OR CJ.IMPORTEPEDAGIO>0 OR CJ.IMPORTESEGURO>0) ");
		sql.append("    AND CJ.CORRIDA_ID = CR.CORRIDA_ID ");
		sql.append("    AND CJ.FECCORRIDA = CR.FECCORRIDA ");
		sql.append("    AND PO.PARADA_ID = CJ.ORIGEN_ID ");
		sql.append("    AND PD.PARADA_ID = CJ.DESTINO_ID ");
		sql.append("    AND PV.PUNTOVENTA_ID = CJ.PUNTOVENTA_ID ");
		sql.append("    AND EM.EMPRESA_ID = CJ.EMPRESACORRIDA_ID ");
		sql.append("    AND CO.CIUDAD_ID = PO.CIUDAD_ID ");
		sql.append("    AND CD.CIUDAD_ID = PD.CIUDAD_ID ");
		sql.append("    AND CJ.INDSTATUSOPERACION = 'F' ");
		sql.append("    AND CJ.INDREIMPRESION = 0 ");
		sql.append("    AND CJ.INDCANCELACION = 0 ");
		sql.append("    AND DECODE(:TIPO_DATA, 1, CJ.FECHORVENTA, 2, CJ.FECCORRIDA) BETWEEN ");
		sql.append("        :DATA_INICIAL AND :DATA_FINAL ");
		sql.append("    AND EM.EMPRESA_ID = NVL(:EMPRESA_ID, EM.EMPRESA_ID) ");

		if (!puntosVentaIds.equals("Todas")) {
			sql.append("    AND PV.PUNTOVENTA_ID IN (").append(puntosVentaIds).append(") ");
		}

		sql.append("    AND PO.PARADA_ID = NVL(:ORIGEN_ID, PO.PARADA_ID) ");
		sql.append("    AND PD.PARADA_ID = NVL(:DESTINO_ID, PD.PARADA_ID) ");

		sql.append("  GROUP BY RT.PREFIXO, ");
		sql.append("           PO.DESCPARADA, ");
		sql.append("           PD.DESCPARADA, ");
		sql.append("           CJ.IMPORTETAXAEMBARQUE, ");
		sql.append("           CJ.IMPORTEPEDAGIO, ");
		sql.append("           CJ.IMPORTESEGURO, ");
		sql.append("           CO.ESTADO_ID, ");
		sql.append("           CD.ESTADO_ID, ");
		sql.append("           EM.NOMBEMPRESA, ");
		sql.append("           EM.EMPRESA_ID, ");
		sql.append("       	   CJ.PUNTOVENTA_ID, ");
		sql.append("           PV.NOMBPUNTOVENTA ");
		sql.append("  ORDER BY EM.NOMBEMPRESA, PO.DESCPARADA, PD.DESCPARADA, PV.NOMBPUNTOVENTA ");

		return sql.toString();
	}
}
