package com.rjconsultores.ventaboletos.relatorios.render;

import java.io.ByteArrayOutputStream;
import java.io.InputStream;
import java.util.Locale;
import java.util.ResourceBundle;

import net.sf.jasperreports.engine.JRDataSource;
import net.sf.jasperreports.engine.JRParameter;
import net.sf.jasperreports.engine.JasperExportManager;
import net.sf.jasperreports.engine.JasperFillManager;
import net.sf.jasperreports.engine.JasperPrint;
import net.sf.jasperreports.engine.export.JExcelApiExporter;
import net.sf.jasperreports.engine.export.JExcelApiExporterParameter;

import com.rjconsultores.ventaboletos.relatorios.utilitarios.Relatorio;
import com.rjconsultores.ventaboletos.relatorios.utilitarios.SaidaRelatorio;
import org.zkoss.util.Locales;

/**
 * Implementao da classe abstrata de renderizao acoplada a uma tecnologia de renderizao especifica
 * 
 * @author Bruno H. G. Gouva <bruno@rjconsultores.com.br>
 * 
 */
public class RenderRelatorioJasper {

	protected JasperPrint jasperPrint;

	private Relatorio relatorio;

	/**
	 * @param relatorio
	 *            Enum do tipo Relatorio contendo as configuraes de gerao de todos os relatrios
	 * @param connection
	 *            Objeto de conexo do banco que ser utilizado para gerao do relatrio, s deve ser passado se no for utilizado nenhum tipo de Custom Data Source
	 * 
	 * @throws Exception
	 *             Excesso contendo regra de utilizao da classe
	 */
	public RenderRelatorioJasper(Relatorio relatorio) throws Exception {
		this.relatorio = relatorio;
	}

	public void preRender() {
		this.initBundler();
		this.initLocale();

	}

	private void initBundler() {
		// TODO Alterar para injeo de acordo com usurio logado ou outro meio para obter o Locale correto
		
		Locale locale = Locales.getCurrent(); //new Locale("pt", "BR");
		try {

			ResourceBundle resource = ResourceBundle.getBundle("com.rjconsultores.ventaboletos.relatorios.internacionalizacao." + this.relatorio.getNome(), locale);
			this.relatorio.getParametros().put(JRParameter.REPORT_RESOURCE_BUNDLE, resource);
		} catch (Exception e) {
		}
	}

	private void initLocale() {
		// TODO Alterar para injeo de acordo com usurio logado ou outro meio para obter o Locale correto
		Locale locale = new Locale("pt", "BR");
		this.relatorio.getParametros().put(JRParameter.REPORT_LOCALE, locale);
	}

	protected InputStream getTemplateInputStream() throws Exception {
		String diretorio = "/com/rjconsultores/ventaboletos/relatorios/templates/" + this.relatorio.getNome() + ".jasper";
		InputStream iStemplate = this.getClass().getResourceAsStream(diretorio);

		if (iStemplate == null) {
			throw new Exception("No foi possivel localizar o template do reltorio no diretorio a seguir: " + diretorio);
		}

		return iStemplate;

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.rjconsultores.ventaboletos.relatorios.utilitarios.IRenderRelatorio#render()
	 */

	public byte[] render(SaidaRelatorio saida) throws Exception {
		this.preRender();

		byte[] conteudo = null;
		
		if (this.jasperPrint == null) {

			if (this.relatorio.isCollectionDataSource()) {
				this.jasperPrint = JasperFillManager.fillReport(this.getTemplateInputStream(), this.relatorio.getParametros(), this.relatorio.getCollectionDataSource());
			} else {
				if (this.relatorio.isCustomDataSource())
					this.jasperPrint = JasperFillManager.fillReport(this.getTemplateInputStream(), this.relatorio.getParametros(), (JRDataSource) this.relatorio.getCustomDataSource());
				else
					this.jasperPrint = JasperFillManager.fillReport(this.getTemplateInputStream(), this.relatorio.getParametros(), this.relatorio.getConexao());
			}
		}

		switch (saida) {

		case PDF:
			conteudo = this.renderPdf();
			break;

		case XLS:
			conteudo = this.renderXls();
			break;

		}

		return conteudo;
	}

	protected byte[] renderXls() throws Exception {

		JExcelApiExporter exporterXLS = new JExcelApiExporter();
		ByteArrayOutputStream output = new ByteArrayOutputStream();
		exporterXLS.setParameter(JExcelApiExporterParameter.JASPER_PRINT, this.jasperPrint);
		exporterXLS.setParameter(JExcelApiExporterParameter.OUTPUT_STREAM, output);
		exporterXLS.setParameter(JExcelApiExporterParameter.IS_ONE_PAGE_PER_SHEET, Boolean.FALSE);
		exporterXLS.setParameter(JExcelApiExporterParameter.IS_DETECT_CELL_TYPE, Boolean.TRUE);
		exporterXLS.setParameter(JExcelApiExporterParameter.IS_WHITE_PAGE_BACKGROUND, Boolean.FALSE);
		exporterXLS.setParameter(JExcelApiExporterParameter.IS_REMOVE_EMPTY_SPACE_BETWEEN_ROWS, Boolean.TRUE);

		// collapseRowSpan
		// JRXlsExporterParameter.
		exporterXLS.exportReport();
		return output.toByteArray();

	}

	protected byte[] renderPdf() throws Exception {

		return JasperExportManager.exportReportToPdf(this.jasperPrint);

	}

}
